/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.engine.orchestra;

import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.wsdl.Definition;
import javax.wsdl.Operation;
import javax.wsdl.PortType;
import javax.xml.namespace.QName;

import org.jbpm.pvm.env.Environment;
import org.jbpm.pvm.env.EnvironmentFactory;
import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.wsdl.WsdlUtil;

/**
 * This class implements an environment for a BPEL process
 * @author Adrien LOUIS - eBMWebSourcing
 * @author Nicolas SALATGE - eBMWebSourcing
 *
 */
public final class BPELProcessEnvironment {

	/**
	 * BPEL process
	 */
	public BpelProcess bpelProcess;

	/**
	 * The jbi environment for orchestra
	 */
	public Environment jbiEnvironment;

	public Environment getJbiEnvironment() {
        return jbiEnvironment;
    }

    /**
	 * the environment Factory
	 */
	public final EnvironmentFactory environmentFactory;

	/**
	 * BPEL process url
	 */
	public final URL bpelProcessURL;

	public final String bpelProcessName;
	/**
	 * WSDL url
	 */
	public final URL wsdlURL;

	/**
	 * WSDL Definition
	 */
	public final Definition wsdlDef;

	/**
	 * Externals WSDLs
	 */
	public final List<URL> externalsWsdls;

	/**
	 * Default constructor
	 *
	 * @param bpelProcessName
	 *            the URL of bpel process
	 * @param wsdlName
	 *            the URL of wsdl
	 * @param environmentFactory
	 *            the jbi environnement factory
	 * @param externalsWsdls
	 *            the externals wsdl
	 */
	public BPELProcessEnvironment(final URL bpelProcessName, final URL wsdlName,
			final EnvironmentFactory environmentFactory,final ArrayList<URL> externalsWsdls) {

		// Initialization
		this.environmentFactory = environmentFactory;

		this.bpelProcessURL = bpelProcessName;
		this.wsdlURL = wsdlName;
		this.wsdlDef = WsdlUtil.readWsdl(this.wsdlURL);
		this.bpelProcess = null;
		this.externalsWsdls = externalsWsdls;
		this.bpelProcessName = BPELProcessEnvironment.extractProcessNameFromURL(bpelProcessURL);
		this.openContext();

		// this.invocations = new ArrayList<ServiceInvocation>();
	}


/**
 * Extract the process name from the url :
 * the last part of the url (after the last '/')
 * @param processURL can be null
 * @return process name, null if url is null or have no name
 */
private static String extractProcessNameFromURL(URL processURL){
	String result = null;
	if (processURL != null){
		String path = processURL.getPath();
		int index = path.lastIndexOf("/");
		index = (index>0)?index:0;
		result = path.substring(index);
	}
	return result;
}


	/**
	 * Return true if the operation corresponding to the given parameters is a
	 * requestResponse operation.
	 *
	 * @param bpelProcess1
	 *            bpelProcess
	 * @param portTypeQName
	 *            portTypeName
	 * @param operationName
	 *            operationName
	 * @return true if the operation corresponding to the given parameters is a
	 *         requestResponse operation
	 * @throws OrchestraException
	 */
	@SuppressWarnings("unchecked")
	public boolean isRequestResponse(final BpelProcess bpelProcess1, final QName portTypeQName,
			final String operationName) throws OrchestraException {
		final PortType pt = bpelProcess1.getWsdlInfos().getPortType(portTypeQName);
		if (pt == null) {
			throw new OrchestraException("Process '" + bpelProcess1.getName() + "', portType '"
					+ portTypeQName + "' not found : cannot define if it is a request/response"
					+ " call or a oneWay call.");
		}
		final List<Operation> operations = pt.getOperations();
		for (final Operation op : operations) {
			if (op.getName().equals(operationName)) {
				return op.getOutput() != null;
			}
		}
		throw new OrchestraException("Operation '" + operationName + "' not found in portType '"
				+ portTypeQName
				+ "' : cannot define if it is a request/response call or a oneWay call.");
	}

	/**
	 * Open the JBI context
	 */
	public void openContext() {
		this.jbiEnvironment = this.environmentFactory.openEnvironment();
	}

	/**
	 * Close the context
	 */
	public void closeContext() {
		this.jbiEnvironment.close();
		this.bpelProcess = null;
	}

}
