/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.engine.orchestra;

import java.io.File;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;

import javax.jbi.JBIException;

import org.jbpm.pvm.env.EnvironmentFactory;
import org.ow2.orchestra.env.EnvFactoryRepository;
import org.ow2.orchestra.services.impl.DbRepository;
import org.ow2.orchestra.services.itf.Invoker;
import org.ow2.orchestra.services.itf.Publisher;
import org.ow2.orchestra.services.itf.Repository;
import org.ow2.petals.component.framework.se.AbstractServiceEngine;
import org.ow2.petals.engine.orchestra.jbi.JBIEnvXmlGenerator;
import org.ow2.petals.engine.orchestra.jbi.JBIInvoker;
import org.ow2.petals.engine.orchestra.jbi.JBIPublisher;
import org.ow2.petals.engine.orchestra.jbi.JBIReceiver;
import org.ow2.petals.engine.orchestra.util.LoggerHelper;
import org.ow2.petals.engine.orchestra.util.Marshaller;

/**
 * The Orchestra Service Engine component class.
 * @author Adrien LOUIS - eBMWebSourcing
 * @author Nicolas SALATGE - eBM WebSourcing
 *
 */
public class OrchestraEngine extends AbstractServiceEngine {

	/**
	 * database name.
	 */
	private static String DB_NAME = "hsqldb";


	/**
	 * Map containing all deployed bpelProcesses
	 */
	private Map<String,BPELProcessEnvironment>  bpelProcessMap;

	/**
	 * the environment Factory
	 */
	private EnvironmentFactory environmentFactory;

	/**
	 * The default constructor
	 *
	 */
	public OrchestraEngine() {
		super();
	}

	public Map<String, BPELProcessEnvironment> getBPELProcessEnvironmentMap() {
		return bpelProcessMap;
	}

	/**
	 * initialize the component
	 *
	 * @throws JBIException
	 *             impossible to initialize the component
	 */
	@Override
	protected final void doInit() throws JBIException {
		super.doInit();

		this.getLogger().log(Level.INFO, "Orchestra Intialization...");

		// FIXME store loggers handlers before orchestra core initialization
		LoggerHelper loggerHelper = new LoggerHelper(getLogger());
		loggerHelper.storeLoggers();

		// create the BPELProcessEnvironment map
		this.bpelProcessMap = new HashMap<String, BPELProcessEnvironment>();

		//  Create the jbi environment for orchestra
		this.environmentFactory = this.createEnvironmentFactory();

		// set service unit listener
		final OrchestraSUListener suListener = new OrchestraSUListener(
				bpelProcessMap, this.environmentFactory, this.getLogger());
		this.setServiceUnitListener(suListener);

		// FIXME reset handlers and level
		// set levels of orchestra sub elements loggers
		loggerHelper.resetLoggers();
	}


	/**
	 * Create and initialize a JBI environment for orchestra
	 *
	 *
	 * @return the jbi environment
	 */
	public final EnvironmentFactory createEnvironmentFactory() {
		final JBIEnvXmlGenerator envGenerator = new JBIEnvXmlGenerator(this
				.getContext().getWorkspaceRoot()
				+ File.separator + DB_NAME);

		// final DefaultEnvXMLGenerator envGenerator = new
		// DefaultEnvXMLGenerator();
		envGenerator.setApplicationEntry(Publisher.DEFAULT_KEY,
				JBIPublisher.class);
		envGenerator.setApplicationEntry(Repository.DEFAULT_KEY,
				DbRepository.class);
		envGenerator.setApplicationEntry(Invoker.DEFAULT_KEY, JBIInvoker.class);

		JBIInvoker.init(getContext(), getChannel(),getLogger());
		JBIReceiver.setLogger(getLogger());
		Marshaller.getInstance().init(getChannel().createExchangeFactory());
		final EnvironmentFactory environmentFac = envGenerator.createEnvironmentFactory();
		
		EnvFactoryRepository.set(environmentFac);

		return environmentFac;
	}

}
