/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.engine.orchestra.jbi;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.security.auth.Subject;
import javax.wsdl.Operation;
import javax.wsdl.Part;
import javax.wsdl.PortType;
import javax.wsdl.WSDLException;
import javax.xml.namespace.QName;

import org.jbpm.pvm.env.Environment;
import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.runtime.PartnerLinkRuntime;
import org.ow2.orchestra.runtime.VariableRuntime;
import org.ow2.orchestra.services.OperationKey;
import org.ow2.orchestra.services.itf.Invoker;
import org.ow2.orchestra.services.itf.Repository;
import org.ow2.orchestra.var.Message;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.engine.orchestra.OrchestraEngineException;
import org.ow2.petals.engine.orchestra.util.Marshaller;
import org.ow2.petals.engine.orchestra.util.Sender;

/**
 * This class implements the JBI Invoker for Orchestra
 * @author alouis
 * @author Nicolas SALATGE - eBMWebSourcing
 */
public class JBIInvoker implements Invoker {

	private static Logger logger;

	private static DeliveryChannel channel;

	private static Sender sender;

	/**
	 * The security context
	 */
	private Subject securitySubject;

	/**
	 * Default constructor
	 */
	public JBIInvoker() {
		super();
		this.securitySubject = null;
	}

	/**
	 * Init the JBI INvoker
	 *
	 * @param e
	 *            the engine component
	 */
	public static final void init(final ComponentContext context,  final DeliveryChannel deliveryChannel, final Logger log) {
		logger = log;
		channel = deliveryChannel;
		logger.log(Level.FINEST, "Creation of Sender");
		sender = new Sender(context, channel,logger);

	}

	/**
	 * Perform an invoke using the given parameters.
	 *
	 * @param operationKey
	 *            the key operation
	 * @param name
	 *            the name of the invoke activity
	 * @param partnerLinkRuntime
	 *            the partnerLink runtime value of the invoke
	 * @param variableRuntime
	 *            the value of the variable used in invoke
	 * @return the message received in response of this call (if there is
	 *         one...)
	 */
	public final Message invoke(final OperationKey operationKey, final String name,
			final PartnerLinkRuntime partnerLinkRuntime, final VariableRuntime variableRuntime) {
		Message messageReturned = null;

		try {
			logger.log(Level.FINEST, "Entering in jbi invoke method");

			// We find the portType specified for the partner role
			final QName portTypeQName = operationKey.getPortTypeQName();
			final String operationName = operationKey.getOperationName();
			final QName processQName = operationKey.getProcessQName();

			final Repository repository = Environment.getCurrent().get(Repository.class);
			final BpelProcess process = repository.getProcess(processQName);


			// ProcessRepository processRepository =
			// CurrentEnvironment.get().get(Repository.class).getProcessRepository();
			// final BpelProcess process = CurrentEnvironment.get().get(Repository.class).getProcess(
			//		processQName);

			final PortType portType = process.getWsdlInfos().getPortType(portTypeQName);
			if(portType == null) {
				throw new org.ow2.orchestra.exception.OrchestraException("portType "
						+ portTypeQName + " not found");
			}

			// Get the Operation associated with the portType.



			final Operation operation = portType.getOperation(operationName, null, null);
			if (operation == null) {
				throw new org.ow2.orchestra.exception.OrchestraException("Operation "
						+ operationName + " not found");
			}

			logger.log(Level.FINEST, "Operation name: " + operation.getName());

			logger.log(Level.FINEST,
					"partnerLinkRuntime name: " + partnerLinkRuntime.getName());


			final Message message = (Message) variableRuntime.getValue();

			logger.log(Level.FINEST,
					"Bpel message corresponding to the jbi request: " + message);

			Exchange request = null;
			@SuppressWarnings("unchecked")
			List<Part> values= operation.getInput().getMessage().getOrderedParts(operation.getParameterOrdering());

			request = Marshaller.getInstance().createJBIMessageExchangeFromBPELMessage(
					process.getWsdlInfos(), portTypeQName, message, operation,
					values, portType.getQName(), this.getSecuritySubjet());



			messageReturned = sender.send(partnerLinkRuntime.getPartnerRoleEndPointReference(), request, null, process.getWsdlInfos(), portTypeQName, operation);

			logger.log(Level.FINEST, "Exit of the jbi invoke method");

		} catch (final WSDLException e) {
			logger.log(Level.SEVERE,e.getMessage(),e);
			throw new org.ow2.orchestra.exception.OrchestraException("A WSDL exception was caught : ", e);
		} catch (final OrchestraEngineException e) {
			logger.log(Level.SEVERE,e.getMessage(),e);
			throw new org.ow2.orchestra.exception.OrchestraException(
					"Exception caught while invoke JBI Service", e);
		} 
		return messageReturned;
	}

	/**
	 * Get the security subject
	 */
	public final Subject getSecuritySubjet() {
		return this.securitySubject;
	}

	/**
	 * Set the security subject
	 *
	 * @param securitySubject1
	 */
	public final void setSecuritySubjet(final Subject securitySubject1) {
		this.securitySubject = securitySubject1;
	}

}
