/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.engine.orchestra.jbi;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import javax.jbi.messaging.MessagingException;
import javax.wsdl.Operation;
import javax.wsdl.Part;
import javax.wsdl.PortType;
import javax.xml.namespace.QName;

import org.jbpm.pvm.env.Transaction;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.services.MessageCarrier;
import org.ow2.orchestra.services.OperationKey;
import org.ow2.orchestra.services.Receiver;
import org.ow2.orchestra.services.itf.Invoker;
import org.ow2.orchestra.var.Message;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.engine.orchestra.BPELProcessEnvironment;
import org.ow2.petals.engine.orchestra.OrchestraEngineException;
import org.ow2.petals.engine.orchestra.util.Marshaller;
import org.w3c.dom.Document;

/**
 * implementation for the services to be published for a deployed bpel process.
 *
 * @author alouis
 */
public class JBIReceiver extends Receiver {

	private static Logger logger;

	public JBIReceiver() {
		super();
		// TODO Auto-generated constructor stub
	}

	public static void setLogger(Logger log) {
		logger = log;
	}

	/**
	 * TODO used Start or continue the bpel process with the given jbi
	 * messageechange
	 *
	 * @param exchange
	 *            the message exchange
	 * @throws MessagingException
	 *             problem with JBI MSgExchange access
	 * @throws OrchestraEngineException
	 *             problem while retrieving information for calling a process
	 */
	@SuppressWarnings("unchecked")
    public static void onJBIMessageReceived(
			final BPELProcessEnvironment bpelProcessEnv, final Exchange exchange)
			throws MessagingException, OrchestraEngineException {
		logger.finest("send the received jbm message to the Orchestra server");

		if (bpelProcessEnv.bpelProcess == null) {
			throw new OrchestraEngineException("the Bpel process cannot be null");
		}
		if (exchange == null) {
			throw new OrchestraEngineException("no exchange");
		}

		// Initialization of the JBI invoker
		// TODO: I think that We must realize a clone of JBIInvoker

		
		final JBIInvoker serviceInvoker = (JBIInvoker) bpelProcessEnv.getJbiEnvironment().get(Invoker.class);
		serviceInvoker.setSecuritySubjet(exchange.getInMessage()
				.getSecuritySubject());

		logger.finest("called endpoint service: "
				+ exchange.getEndpoint().getServiceName());
		logger.finest("called operation: " + exchange.getOperation());

		if (exchange.getEndpoint().getInterfaces() == null
				|| exchange.getEndpoint().getInterfaces().length == 0) {
			throw new OrchestraEngineException("can not find the interface");
		}
		final QName portTypeQName = exchange.getEndpoint().getInterfaces()[0];
		final QName processQName = exchange.getEndpoint().getServiceName();

		final String operationName = exchange.getOperation().getLocalPart();

		// Get the input message
		final PortType port = bpelProcessEnv.wsdlDef.getPortType(portTypeQName);

		if (port == null) {
			throw new OrchestraEngineException(
					"the following wsdl port can not be retrieved : "
							+ portTypeQName);
		}

		final Operation operation = port
				.getOperation(operationName, null, null);

		MessageCarrier messageCarrier = null;

		// To retrieve the process to use with this exchange, we compare the
		// interface of the requested service with all the processes name, as
		// the process name must be the interface name
		final OperationKey operationKey = new OperationKey(processQName,
				portTypeQName, operationName);

		if (operation == null) {
			throw new OrchestraEngineException("the operation (" + operationName
					+ ") does not exist in WSDL document: "
					+ bpelProcessEnv.wsdlURL);
		}

		final QName inputMessageQName = new QName(operation.getInput()
				.getMessage().getQName().getNamespaceURI(), operation
				.getInput().getMessage().getQName().getLocalPart());
		logger.finest("processQName: " + processQName);
		logger.finest("portTypeQName: " + portTypeQName);
		final javax.wsdl.Message inputMessage = bpelProcessEnv.bpelProcess
				.getWsdlInfos().getMessage(inputMessageQName);
		logger.finest("received jbi message: " + inputMessage);

		// Convert the JBI message to BPEL message
		final Message incomingMessage = Marshaller.getInstance()
				.buildBPELMessageFromJBIMessage(
						bpelProcessEnv.bpelProcess.getWsdlInfos(),
						portTypeQName, operation,
						inputMessage.getParts().values(),
						exchange.getInMessageContentAsDocument());
		logger.finest("corresponding bpel message: " + incomingMessage);

		try {
			final boolean requestResponse = bpelProcessEnv.isRequestResponse(
					bpelProcessEnv.bpelProcess, portTypeQName, operationName);

			logger.finest("is requestResponse: " + requestResponse);
			if (bpelProcessEnv.isRequestResponse(bpelProcessEnv.bpelProcess,
					portTypeQName, operationName)) {
				messageCarrier = new MessageCarrier();
			}
		} catch (final org.ow2.orchestra.exception.OrchestraException e) {
			if (bpelProcessEnv.jbiEnvironment != null) {
				final Transaction t = bpelProcessEnv.jbiEnvironment
						.getTransaction();
				if (t != null) {
					t.setRollbackOnly();
				}
			}
			throw new OrchestraEngineException(e);
		}

		logger.finest("handle process...");
		final BpelExecution bpelExecution = Receiver.handle(incomingMessage, operationKey, bpelProcessEnv.environmentFactory, messageCarrier);
		logger.finest("process finished...");
		BpelExecution instance = null;
		if (bpelExecution != null) {
			instance = bpelExecution.getProcessInstance();
		}
		logger.finest("instance of process: " + instance);

		// Get the response if exist
		Message responseMessage = null;

		if (messageCarrier != null) {
			responseMessage = messageCarrier.getMessage();
			logger.finest("responseMessage: " + responseMessage);
		}

		if (responseMessage != null) {
			logger.finest("bpel response message = " + responseMessage);

			// We make the response
			List<Part> values = operation.getOutput().getMessage()
					.getOrderedParts(operation.getParameterOrdering());
			if (((values != null) && (values.size() == 0)) || (values == null)) {
				Map outputParts = operation.getOutput().getMessage().getParts();
				values = new ArrayList<Part>(outputParts.values());
			}

			final Document jbiDocument = Marshaller.getInstance()
					.buildJBIMessageFromBPELMessage(
							bpelProcessEnv.bpelProcess.getWsdlInfos(),
							portTypeQName, responseMessage, operation, values,
							port.getQName().getNamespaceURI());

			if (exchange.isInOutPattern() || exchange.isInOptionalOutPattern()) {
				exchange.setOutMessageContent(jbiDocument);
			} else {
				throw new MessagingException(
						"The process try to set a response, but the Exchange pattern does not allow it.");
			}
		}

	}
}
