/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.engine.orchestra.util;

import java.io.ByteArrayInputStream;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.Logger;

/**
 *
 * @author alouis
 *
 */
public class LoggerHelper {

	HashMap<String, Handler[]> handlersMap = new HashMap<String, Handler[]>();
	HashMap<String, Level> levelsMap = new HashMap<String, Level>();
	HashMap<String, Logger> loggersMap = new HashMap<String, Logger>();

	Logger _logger;

	public LoggerHelper(Logger logger) {
		this._logger = logger;
	}

	public void storeLoggers() {
		_logger
				.log(Level.FINEST,
						"FIX : store all existing loggers before orchestra server initialization");

		Enumeration<String> loggersName = LogManager.getLogManager()
				.getLoggerNames();
		for (String loggerName = loggersName.nextElement(); loggersName
				.hasMoreElements(); loggerName = loggersName.nextElement()) {
			Logger log = LogManager.getLogManager().getLogger(loggerName);
			handlersMap.put(loggerName, log.getHandlers());
			levelsMap.put(loggerName, log.getLevel());
			loggersMap.put(loggerName, log);
		}
	}

	public void resetLoggers() {

		LogManager.getLogManager().reset();
		try {
			LogManager.getLogManager().readConfiguration(
					new ByteArrayInputStream(".level = SEVERE".getBytes()));
		} catch (Exception e) {
			e.printStackTrace();
		}

		// reset the component logger
		String componentLoggerName = this._logger.getName();
		Logger newComponentLogger = LogManager.getLogManager().getLogger(
				componentLoggerName);
		recopyLogger(_logger, newComponentLogger);
		_logger = newComponentLogger;

		_logger.log(Level.FINEST, "all loggers has been cleaned");

		// reset other loggers
		for (Logger oldLogger : loggersMap.values()) {
			Logger newLogger = LogManager.getLogManager().getLogger(
					oldLogger.getName());
			recopyLogger(oldLogger, newLogger);

		}
		_logger.log(Level.FINEST, "previously existing loggers has been reset");

		// set orchestra loggers to the same values than this component logger
		Enumeration<String> loggersName = LogManager.getLogManager()
				.getLoggerNames();
		while (loggersName.hasMoreElements()) {
			String tmpLoggerName = loggersName.nextElement();
			Logger tmpLogger = LogManager.getLogManager().getLogger(
					tmpLoggerName);
			if (tmpLoggerName.startsWith("org.ow2.orchestra")) {
				// the logger is an orchestra server logger, set its level as
				// this component logger one
				// let the org.jbpm
				configureLogger(tmpLogger, levelsMap.get(componentLoggerName),
						handlersMap.get(componentLoggerName));
				tmpLogger.setParent(_logger);
			}
		}
		_logger
				.log(
						Level.FINEST,
						"orchestra server loggers are now inheriting from the petals-se-orchestra logger");
	}

	private void recopyLogger(Logger oldLogger, Logger newLogger) {
		newLogger.setFilter(oldLogger.getFilter());
		newLogger.setLevel(oldLogger.getLevel());
		
		if(oldLogger.getParent() != null) {
		    newLogger.setParent(oldLogger.getParent());
		}
		newLogger.setUseParentHandlers(oldLogger.getUseParentHandlers());
		for (Handler handler : newLogger.getHandlers()) {
			// remove handlers
			newLogger.removeHandler(handler);
		}
		for (Handler handler : oldLogger.getHandlers()) {
			// remove handlers
			newLogger.addHandler(handler);
		}
		// use the stored Level and handlers
		configureLogger(newLogger, levelsMap.get(oldLogger.getName()),
				handlersMap.get(oldLogger.getName()));
	}

	/**
	 * remove handlers from this logger and add the new ones set the level
	 *
	 * @param log
	 * @param level
	 * @param handlers
	 */
	private void configureLogger(Logger log, Level level, Handler[] handlers) {
		for (Handler oldHandler : log.getHandlers()) {
			// remove handlers
			log.removeHandler(oldHandler);
		}

		for (Handler handler : handlers) {
			// reset old handlers
			log.addHandler(handler);
		}

		log.setLevel(level);
	}

}
